__author__ = 'Milinda Pathirage'

import os
import io
import urllib
import urllib2
import tempfile
import utils


class DataAPIClient:

    def __init__(self, endpoint=None, client_id=None, client_secret=None, token_endpoint=None):
        if endpoint is None:
            self.endpoint = os.environ['HTRC_IPYTHON_DATAAPI_EPR']
        else:
            self.endpoint = endpoint

        if client_id is None:
            self.client_id = os.environ['HTRC_IPYTHON_OAUTH2_CLIENT_ID']
        else:
            self.client_id = client_id

        if client_secret is None:
            self.client_secret = os.environ['HTRC_IPYTHON_OAUTH2_CLIENT_SECRET']
        else:
            self.client_secret = client_secret

        if token_endpoint is None:
            self.token_endpoint = os.environ['HTRC_IPYTHON_OAUTH2_TOKEN_ENDPOINT']
        else:
            self.token_endpoint = token_endpoint

    def get_volumes(self, volume_list, access_token=None):
        if access_token is None:
            access_token = utils.get_oauth2_token(self.token_endpoint, self.client_id, self.client_secret)

        headers = {"Authorization": "Bearer " + access_token,
                   "Content-type": "application/x-www-form-urlencoded"}

        volume_id_list = '|'.join(volume_list)
        parameters = {'volumeIDs': volume_id_list}
        body = urllib.urlencode(parameters)
        request_epr = self.endpoint + "/volumes"

        req = urllib2.Request(request_epr, body, headers)

        response = urllib2.urlopen(req)

        if response.code != 200:
            raise urllib2.HTTPError(response.url, response.code, response.read(), response.info(), response.fp)

        zip_content = io.BytesIO(response.read())

        temp_dir = tempfile.mkdtemp()

        utils.unzip(zip_content, temp_dir)

        return temp_dir





__author__ = 'Milinda Pathirage'

import urllib
import urllib2
import json
import zipfile
import os


def get_oauth2_token(token_endpoint, client_id, client_secret):
    headers = {'content-type': 'application/x-www-form-urlencoded'}

    # request body
    values = {'grant_type': 'client_credentials',
              'client_id': client_id,
              'client_secret': client_secret}

    body = urllib.urlencode(values)

    # request method must be POST
    req = urllib2.Request(token_endpoint, body, headers)
    try:
        # urllib2 module sends HTTP/1.1 requests with Connection:close header included
        response = urllib2.urlopen(req)

        # any other response code means the OAuth2 authentication failed. raise exception
        if response.code != 200:
            raise urllib2.HTTPError(response.url, response.code, response.read(), response.info(), response.fp)

        # response body is a JSON string
        oauth2_token_response_string = response.read()

        # parse JSON string using python built-in json lib
        oauth2_token_response_json = json.loads(oauth2_token_response_string)

        # return the access token
        return oauth2_token_response_json["access_token"]

    # response code in the 400-599 range will raise HTTPError
    except urllib2.HTTPError as e:
        # just re-raise the exception
        raise Exception(str(e.code) + " " + str(e.reason) + " " + str(e.info) + " " + str(e.read()))


def unzip(zip_content, dest_dir):
    # From http://stackoverflow.com/a/12886818
    with zipfile.ZipFile(zip_content, "r") as zf:
        for member in zf.infolist():
            # Path traversal defense copied from
            # http://hg.python.org/cpython/file/tip/Lib/http/server.py#l789
            words = member.filename.split('/')
            path = dest_dir
            for word in words[:-1]:
                drive, word = os.path.splitdrive(word)
                head, word = os.path.split(word)

                if word in (os.curdir, os.pardir, ''):
                    continue

                path = os.path.join(path, word)

            zf.extract(member, path)


from setuptools import setup

setup(
    name = 'htrc',
    packages = ['htrc'],
    version = '0.0.3',
    description = 'Set of Python Libraries for HTRC APIs.',
    author='Milinda Pathirage',
    author_email='milinda.pathirage@gmail.com',
    url='https://github.com/milinda/Vessel',
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Development Status :: 1 - Planning',
        'Environment :: Console',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering :: Information Analysis'
    ]
)